@@ -1,9 +1,7 @@
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@python.org
-# Revision: $Revision: 3165 $
-# Date: $Date: 2005-04-05 04:55:06 +0200 (Tue, 05 Apr 2005) $
+# $Id: test_tables.py 5499 2008-01-14 15:52:01Z goodger $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -29,6 +27,23 @@ utf_16_csv = os.path.join(mydir, 'utf-16.csv')
 utf_16_csv_rel = DocutilsTestSupport.utils.relative_path(None, utf_16_csv)
 empty_txt = os.path.join(mydir, 'empty.txt')
 
+unichr_exception = DocutilsTestSupport.exception_data(
+    'unichr(int("9999999999999", 16))')[0]
+if isinstance(unichr_exception, OverflowError):
+    unichr_exception_string = 'code too large (%s)' % unichr_exception
+else:
+    unichr_exception_string = str(unichr_exception)
+
+null_bytes_code = """
+import csv
+import cStringIO
+csv_data = open('%s', 'rb').read()
+csv_file = cStringIO.StringIO(csv_data)
+reader = csv.reader(csv_file)
+reader.next()
+""" % utf_16_csv
+null_bytes_exception = DocutilsTestSupport.exception_data(null_bytes_code)[0]
+
 totest = {}
 
 totest['table'] = [
@@ -76,6 +91,36 @@ totest['table'] = [
                             False
 """],
 ["""\
+.. table::
+
+   ========== ==========
+   Table      without
+   a          title
+   ========== ==========
+""",
+"""\
+<document source="test data">
+    <table>
+        <tgroup cols="2">
+            <colspec colwidth="10">
+            <colspec colwidth="10">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Table
+                    <entry>
+                        <paragraph>
+                            without
+                <row>
+                    <entry>
+                        <paragraph>
+                            a
+                    <entry>
+                        <paragraph>
+                            title
+"""],
+["""\
 .. table:: title with an *error
 
    ======  =====
@@ -105,6 +150,32 @@ totest['table'] = [
         <paragraph>
             Inline emphasis start-string without end-string.
 """],
+["""\
+.. table:: Not a table.
+
+   This is a paragraph.
+""",
+"""\
+<document source="test data">
+    <system_message level="3" line="1" source="test data" type="ERROR">
+        <paragraph>
+            Error parsing content block for the "table" directive: exactly one table expected.
+        <literal_block xml:space="preserve">
+            .. table:: Not a table.
+            \n\
+               This is a paragraph.
+"""],
+["""\
+.. table:: empty
+""",
+"""\
+<document source="test data">
+    <system_message level="2" line="1" source="test data" type="WARNING">
+        <paragraph>
+            Content block expected for the "table" directive; none found.
+        <literal_block xml:space="preserve">
+            .. table:: empty
+"""],
 ]
 
 if not csv:
@@ -544,7 +615,7 @@ u"""\
         <paragraph>
             Error in "csv-table" directive:
             invalid option value: (option: "widths"; value: '10,y,z')
-            invalid literal for int(): y.
+            %s.
         <literal_block xml:space="preserve">
             .. csv-table:: bad column widths
                :widths: 10,y,z
@@ -560,7 +631,7 @@ u"""\
                :widths: 0 0 0
             \n\
                some, csv, data
-"""],
+""" % DocutilsTestSupport.exception_data('int("y")')[1][0]],
 ["""\
 .. csv-table:: good delimiter
    :delim: /
@@ -678,11 +749,11 @@ u"""\
         <paragraph>
             Error in "csv-table" directive:
             invalid option value: (option: "delim"; value: 'U+9999999999999')
-            code too large (long int too large to convert to int).
+            %s.
         <literal_block xml:space="preserve">
             .. csv-table:: bad delimiter
                :delim: U+9999999999999
-"""],
+""" % unichr_exception_string],
 ["""\
 .. csv-table:: bad CSV data
 
@@ -729,14 +800,14 @@ u"""\
     <system_message level="3" line="1" source="test data" type="ERROR">
         <paragraph>
             Error with CSV data in "csv-table" directive:
-            string with NUL bytes
+            %s
         <literal_block xml:space="preserve">
             .. csv-table:: bad encoding
                :file: %s
                :encoding: latin-1
     <paragraph>
         (7- and 8-bit text encoded as UTF-16 has lots of null/zero bytes.)
-""" % utf_16_csv],
+""" % (null_bytes_exception, utf_16_csv)],
 ["""\
 .. csv-table:: good encoding
    :file: %s
@@ -989,6 +1060,17 @@ totest['list-table'] = [
                * - column 1
                  - column 2
 """],
+["""\
+.. list-table:: empty
+""",
+"""\
+<document source="test data">
+    <system_message level="3" line="1" source="test data" type="ERROR">
+        <paragraph>
+            The "list-table" directive is empty; content required.
+        <literal_block xml:space="preserve">
+            .. list-table:: empty
+"""],
 ]
 
 
